using JuMP
using Gurobi

m = Model(solver=GurobiSolver())

A = Dict(1 => "UP", 2 => "DW")
w = ["UP", "DW"]

b = 55000
G = 80000
r = 0.04
q = 0.01

@defVar(m,u0[1:2] >= 0)
@defVar(m,u1[1:2,1:2] >= 0)
@defVar(m,u2[1:2,1:2,1:2] >= 0)
@defVar(m,u3[1:2,1:2,1:2,1:2] >= 0)

#primo stadio
@addConstraint(m,u0[1]+u0[2] == b)

#secondo stadio
@addConstraint(m,1.25*u0[1] + 1.14*u0[2] == u1[1,1] + u1[2,1])
@addConstraint(m,1.06*u0[1] + 1.12*u0[2] == u1[1,2] + u1[2,2])

#terzo stadio
@addConstraint(m,1.25*u1[1,1] + 1.14*u1[2,1] == u2[1,1,1] + u2[2,1,1])
@addConstraint(m,1.25*u1[1,2] + 1.14*u1[2,2] == u2[1,2,1] + u2[2,2,1])
@addConstraint(m,1.06*u1[1,1] + 1.12*u1[2,1] == u2[1,1,2] + u2[2,1,2])
@addConstraint(m,1.06*u1[1,2] + 1.12*u1[2,2] == u2[1,2,2] + u2[2,2,2])

#quarto stadio
@addConstraint(m,1.25*u2[1,1,1] + 1.14*u2[2,1,1] == G - u3[1,1,1,1] + u3[2,1,1,1])
@addConstraint(m,1.25*u2[1,2,1] + 1.14*u2[2,2,1] == G - u3[1,2,1,1] + u3[2,2,1,1])
@addConstraint(m,1.25*u2[1,1,2] + 1.14*u2[2,1,2] == G - u3[1,1,2,1] + u3[2,1,2,1])
@addConstraint(m,1.25*u2[1,2,2] + 1.14*u2[2,2,2] == G - u3[1,2,2,1] + u3[2,2,2,1])
@addConstraint(m,1.06*u2[1,1,1] + 1.12*u2[2,1,1] == G - u3[1,1,1,2] + u3[2,1,1,2])
@addConstraint(m,1.06*u2[1,2,1] + 1.12*u2[2,2,1] == G - u3[1,2,1,2] + u3[2,2,1,2])
@addConstraint(m,1.06*u2[1,1,2] + 1.12*u2[2,1,2] == G - u3[1,1,2,2] + u3[2,1,2,2])
@addConstraint(m,1.06*u2[1,2,2] + 1.12*u2[2,2,2] == G - u3[1,2,2,2] + u3[2,2,2,2])

@setObjective(m,Max,sum{0.125*q*u3[2,i,j,k],i=1:2, j=1:2, k=1:2} - sum{0.125*r*u3[1,i,j,k],i=1:2, j=1:2, k=1:2})

status = solve(m)

U0 = getValue(u0)
U1 = getValue(u1)
U2 = getValue(u2)
U3 = getValue(u3)

println()

println("  ---------------+------------------------")
@printf("\t         |  stock     bond\n")
println("  ---------------+------------------------")
print("\t         | ")
for i = 1:2
	@printf("%9.3f ",U0[i])
end
println()
println("  ---------------+------------------------")

for j = 1:2
	print("\t",A[j],"       | ")
	for i = 1:2
		@printf("%9.3f ",U1[i,j])
	end
	println()
end
println("  ---------------+------------------------")

for j = 1:2
	for h = 1:2
		print("\t",A[j]," ",A[h],"    | ")
		for i = 1:2
			@printf("%9.3f ",U2[i,j,h])
		end
		println()
	end
end
println("  ---------------+------------------------")
@printf("\t         |prestito  deposito\n")
println("  ---------------+------------------------")
for j = 1:2
	for h = 1:2
		for k = 1:2
			print("\t",A[j]," ",A[h]," ",A[k]," | ")
			for i = 1:2
				@printf("%9.3f ",U3[i,j,h,k])
			end
			println()
		end
	end
end
println("  ---------------+------------------------")

nothing
