using JuMP
using Gurobi

m = Model(solver=GurobiSolver())

w = ["UP", "DW"]

b = 55000
G = 80000
r = 0.04
q = 0.01

t = [1.25 1.06;
     1.14 1.12]

@defVar(m,u0[1:2] >= 0)
@defVar(m,u1[1:2,w] >= 0)
@defVar(m,u2[1:2,w,w] >= 0)
@defVar(m,u3[1:2,w,w,w] >= 0)

#primo stadio
@addConstraint(m,u0[1]+u0[2] == b)

#secondo stadio
for j in 1:2
	@addConstraint(m,sum{t[i,j]*u0[i] - u1[i,w[j]],i=1:2} == 0)
end

#terzo stadio
for j in 1:2
for h in 1:2
	@addConstraint(m,sum{t[i,h]*u1[i,w[j]] - u2[i,w[j],w[h]],i=1:2} == 0)
end
end

#quarto stadio
for j = 1:2
for h = 1:2
for k = 1:2
	@addConstraint(m,sum{t[i,k]*u2[i,w[j],w[h]],i=1:2} == G - u3[1,w[j],w[h],w[k]] + u3[2,w[j],w[h],w[k]])
end
end
end
@setObjective(m,Max,sum{0.125*q*u3[2,w[i],w[j],w[k]],i=1:2, j=1:2, k=1:2} - sum{0.125*r*u3[1,w[i],w[j],w[k]],i=1:2, j=1:2, k=1:2})

status = solve(m)

U0 = getValue(u0)
U1 = getValue(u1)
U2 = getValue(u2)
U3 = getValue(u3)

println()

println("  ---------------+------------------------")
@printf("\t         |  stock     bond\n")
println("  ---------------+------------------------")
print("\t         | ")
for i = 1:2
	@printf("%9.3f ",U0[i])
end
println()
println("  ---------------+------------------------")

for j = w
	print("\t",j,"       | ")
	for i = 1:2
		@printf("%9.3f ",U1[i,j])
	end
	println()
end
println("  ---------------+------------------------")

for j = w
	for h = w
		print("\t",j," ",h,"    | ")
		for i = 1:2
			@printf("%9.3f ",U2[i,j,h])
		end
		println()
	end
end
println("  ---------------+------------------------")
@printf("\t         |prestito  deposito\n")
println("  ---------------+------------------------")
for j = w
	for h = w
		for k = w
			print("\t",j," ",h," ",k," | ")
			for i = 1:2
				@printf("%9.3f ",U3[i,j,h,k])
			end
			println()
		end
	end
end
println("  ---------------+------------------------")

nothing
